package jp.deadend.noname.howmm

import android.util.Log

import java.io.File
import java.io.IOException
import java.nio.charset.Charset

const val REMINDER_CACHE_FILE = "reminder.cache"
const val TITLE_CACHE_FILE = "title.cache"

fun saveCacheFile(cacheFile: File, encoding: Charset, newline: String, list: List<GrepItem>) {
    try {
        cacheFile.bufferedWriter(encoding).use { bw ->
            bw.write("# This file is created by howmm.  Please do not edit.")
            bw.write(newline)
            bw.write(System.currentTimeMillis().toString())
            bw.write(newline)

            list.forEach { item ->
                bw.write("${item.file.absolutePath},${item.linenumber},${item.text}")
                bw.write(newline)
            }
        }
    } catch (e: Exception) {
        Log.e("Howmm", "saveCacheFile(): cache file writing error")
        Log.e("Howmm", e.toString())
        cacheFile.delete()
        return
    }

    Trace.d("${list.size} items saved in cache")
}

fun loadCacheFile(cacheFile: File, encoding: Charset): Pair<List<GrepItem>, Long> {
    val lines = try {
        cacheFile.bufferedReader(encoding).readLines().drop(1) // 最初はコメント行
    } catch (e: IOException) {
        Log.e("Howmm", "loadTitles(): cache file loading error")
        Log.e("Howmm", e.toString())
        listOf("0")
    }
    val cacheLastModified = lines[0].toLongOrNull() ?: 0

    val cachedItems = lines.drop(1).mapNotNull {
        val lst = it.split(',')
        if (lst.size == 3) {
            val f = File(lst[0])
            if (f.lastModified() < cacheLastModified) {
                GrepItem(File(lst[0]), lst[1].toInt(), lst[2])
            } else null
        } else null
    }
    Trace.d("${cachedItems.size} items found in cache")

    return Pair(cachedItems, cacheLastModified)
}

fun clearCacheFile(fileName: String) {
    val cacheFile = File(howmmPrefs.rootDir, fileName)
    if (cacheFile.canRead()) { cacheFile.delete() }
}
