package jp.deadend.noname.howmm

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import android.widget.ArrayAdapter
import android.widget.ListView
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.pm.ShortcutInfoCompat
import androidx.core.content.pm.ShortcutManagerCompat
import androidx.core.graphics.drawable.IconCompat

import java.util.ArrayList

class CreateShortcutsActivity : AppCompatActivity() {
    private val openFileActivity =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == Activity.RESULT_OK) {
                result.data?.getStringExtra(FileChooser.KEY_FILEPATH)?.let {
                    val intent = Intent(this, TextEditor::class.java)
                    intent.action = Intent.ACTION_MAIN
                    intent.putExtra(TextEditor.KEY_PATHNAME, it)
                    val shortcut = ShortcutInfoCompat.Builder(this, "id2")
                        .setShortLabel(getString(R.string.shortcut_name_open_file, it))
                        .setIcon(IconCompat.createWithResource(this, R.drawable.icon))
                        .setIntent(intent)
                        .build()
                    setResult(
                        Activity.RESULT_OK,
                        ShortcutManagerCompat.createShortcutResultIntent(this, shortcut)
                    )
                    finish()
                }
            }
        }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setTitle(R.string.activity_title_create_shortcuts)

        val labels = ArrayList<String>()
        labels.add(getString(R.string.label_create_new))
        labels.add(getString(R.string.label_open_file))

        val listView = ListView(this)
        listView.adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, labels)
        listView.setOnItemClickListener { _, _, position, _ ->
            val item = listView.getItemAtPosition(position) as String
            if (item == getString(R.string.label_create_new)) {
                val intent = Intent(this, TextEditor::class.java)
                intent.action = Intent.ACTION_MAIN
                val shortcut = ShortcutInfoCompat.Builder(this, "id1")
                    .setShortLabel(getString(R.string.shortcut_name_new))
                    .setIcon(IconCompat.createWithResource(this, R.drawable.icon))
                    .setIntent(intent)
                    .build()
                setResult(
                    Activity.RESULT_OK,
                    ShortcutManagerCompat.createShortcutResultIntent(this, shortcut)
                )
                finish()
            } else if (item == getString(R.string.label_open_file)) {
                val intent = Intent(this, FileChooser::class.java).apply {
                    putExtra(FileChooser.KEY_MODE, FileChooser.MODE_OPEN)
                    putExtra(FileChooser.KEY_DIRNAME, howmmPrefs.rootDir)
                    putExtra(FileChooser.KEY_FONTSIZE, howmmPrefs.fontSize)
                }
                openFileActivity.launch(intent)
            }
        }
        setContentView(listView)
    }
}
