package jp.deadend.noname.howmm

import android.os.Bundle
import android.view.MenuItem
import androidx.appcompat.app.AppCompatActivity
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.ListView
import android.widget.TextView

class ExtListActivity : AppCompatActivity() {
    private lateinit var mAdapter: ArrayAdapter<String>

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_extlist)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setTitle(R.string.activity_title_extlist)

        mAdapter = ArrayAdapter(this, android.R.layout.simple_list_item_1)
        mAdapter.add(getString(R.string.pref_extlist_add))
        val extList = howmmPrefs.extList
        for (s in extList) { mAdapter.add(s) }

        val listView = findViewById<ListView>(R.id.listView)
        listView.adapter = mAdapter
        listView.emptyView = findViewById<TextView>(R.id.textViewEmpty)
        listView.onItemClickListener = AdapterView.OnItemClickListener {_, _, position, _ ->
            if (position == 0) {
                val dialog = TextInputDialogFragment.newInstance(getString(R.string.pref_extlist_add))
                dialog.setSingleLine(true)
                dialog.setListener(
                    object : TextInputDialogFragment.Listener {
                        override fun onPositiveClick(result: String) {
                            if (result.isNotEmpty() && !contains(result)) {
                                mAdapter.add(result)
                                mAdapter.notifyDataSetChanged()
                            }
                        }
                        override fun onNegativeClick() {}
                    })
                dialog.show(supportFragmentManager, "dialog")
            } else {
                val itemToRemove = mAdapter.getItem(position)
                val dialog = ConfirmationDialogFragment.newInstance(getString(R.string.message_confirm_remove_item, itemToRemove))
                dialog.setListener(
                    object : ConfirmationDialogFragment.Listener {
                        override fun onPositiveClick() {
                            mAdapter.remove(itemToRemove)
                            mAdapter.notifyDataSetChanged()
                        }
                        override fun onNegativeClick() {}
                    })
                dialog.show(supportFragmentManager, "dialog")
            }
        }
    }

    override fun onPause() {
        val exts = mutableListOf<String>()
        for (i in 1 until mAdapter.count) {
            mAdapter.getItem(i)?.let { exts.add(it) }
        }
        howmmPrefs.extList = exts

        super.onPause()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) { finish() }

        return super.onOptionsItemSelected(item)
    }

    private fun contains(s: String): Boolean {
        for (i in 1 until mAdapter.count) {
            if (mAdapter.getItem(i).equals(s, ignoreCase = true)) { return true }
        }

        return false
    }
}