package jp.deadend.noname.howmm

import android.graphics.Paint
import android.graphics.Typeface
import android.os.Parcel
import android.text.ParcelableSpan
import android.text.TextPaint
import android.text.style.MetricAffectingSpan

/**
 *
 * Describes a style in a span.
 * Note that styles are cumulative -- if both bold and italic are set in
 * separate spans, or if the base style is bold and a span calls for italic,
 * you get bold italic.  You can't turn off a style from the base style.
 *
 * Note: bold and italic is always fake.
 */
class FakeBoldItalicSpan : MetricAffectingSpan, ParcelableSpan {
    /**
     * Returns the style constant defined in [android.graphics.Typeface].
     */
    val style: Int

    /**
     *
     * @param style An integer constant describing the style for this span. Examples
     * include bold, italic, and normal. Values are constants defined
     * in [android.graphics.Typeface].
     */
    constructor(style: Int) {
        this.style = style
    }

    constructor(src: Parcel) {
        style = src.readInt()
    }

    override fun getSpanTypeId(): Int {
        return 7
        //~ return TextUtils.STYLE_SPAN;
    }

    override fun describeContents() = 0

    override fun writeToParcel(dest: Parcel, flags: Int) {
        dest.writeInt(style)
    }

    override fun updateDrawState(ds: TextPaint) {
        fake_apply(ds, style)
    }

    override fun updateMeasureState(paint: TextPaint) {
        fake_apply(paint, style)
    }

    private fun fake_apply(paint: Paint, style: Int) {
        val old = paint.typeface
        val oldStyle =  old?.style ?: 0

        val want = oldStyle or style

        val tf = if (old == null) {
            Typeface.defaultFromStyle(want)
        } else {
            Typeface.create(old, want)
        }

        if (style and Typeface.BOLD != 0) {
            paint.isFakeBoldText = true
        }

        if (style and Typeface.ITALIC != 0) {
            paint.textSkewX = -0.25f
        }

        paint.typeface = tf
    }
}