package jp.deadend.noname.howmm

import android.content.Context
import android.text.SpannableString
import android.util.AttributeSet
import android.util.TypedValue
import android.view.KeyEvent
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.TextView

class GrepView : ScrollInhibitableListView {
    private var mCallback: Callback? = null

    interface Callback {
        fun onGrepItemClicked(position: Int)
        fun onGrepItemLongClicked(position: Int): Boolean
    }

    constructor(context: Context) : super(context)
    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)
    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

    init {
        isSmoothScrollbarEnabled = true
        isScrollingCacheEnabled = true
        isFastScrollEnabled = true
        dividerHeight = 2
        onItemClickListener = OnItemClickListener { _, _, position, _ ->
            mCallback?.onGrepItemClicked(position)
        }
//        onItemSelectedListener = object : OnItemSelectedListener {
//            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
//                mCallback?.onGrepItemClicked(position)
//            }
//            override fun onNothingSelected(parent: AdapterView<*>?) {}
//        }
        onItemLongClickListener = OnItemLongClickListener { _, _, position, _ ->
            mCallback?.onGrepItemLongClicked(position) ?: false
        }

    }

    fun setCallback(cb: Callback) {
        mCallback = cb
    }

    override fun dispatchKeyEvent(event: KeyEvent): Boolean {
        val action = event.action
        val keycode = event.keyCode
        if (action == KeyEvent.ACTION_UP || action == KeyEvent.ACTION_DOWN) {
            if (keycode == KeyEvent.KEYCODE_SPACE || keycode == KeyEvent.KEYCODE_PAGE_DOWN || keycode == KeyEvent.KEYCODE_PAGE_UP) {
                return true
            }
        }

        return super.dispatchKeyEvent(event)
    }

    open class GrepAdapter(
            context: Context,
            objects: List<GrepItem>
    ) : ArrayAdapter<GrepItem>(context, 0, objects) {
        private var mRegex: Regex? = null
        private var mFontSize: Int = 0
        private val mPadding = TypedValue.applyDimension(
                TypedValue.COMPLEX_UNIT_DIP, PADDING, context.resources.displayMetrics
        ).toInt()

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            val tv = convertView as TextView?
                    ?: TextView(context).apply {
                        textSize = mFontSize.toFloat()
                        gravity = android.view.Gravity.CENTER_VERTICAL
                        setPadding(mPadding, mPadding, 0, mPadding)
                        setTextColor(-0x1000000)
                    }

            getItem(position)?.let { tv.text = createSpannedText(it) }

            return tv
        }

        open fun createSpannedText(item: GrepItem): SpannableString {
            return TextSpanner.spanGrepAdapter(item.text, mRegex)
        }

        fun setFormat(regex: Regex?, size: Int) {
            mRegex = regex
            mFontSize = size
        }

        companion object {
            private const val PADDING = 4.0f
        }
    }

    class ReminderAdapter(context: Context, objects: List<GrepItem>) : GrepAdapter(context, objects) {
        override fun createSpannedText(item: GrepItem): SpannableString {
            val rItem = item.reminder
            return if (rItem != null) {
                TextSpanner.spanReminderAdapter(item.text, rItem.type, rItem.priority)
            } else {
                SpannableString("")
            }
        }
    }
}
