package jp.deadend.noname.howmm

import android.app.PendingIntent
import android.appwidget.AppWidgetManager
import android.appwidget.AppWidgetProvider
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.text.SpannableString
import android.widget.RemoteViews
import android.util.Log
import kotlinx.coroutines.*
import java.io.File

private fun calcNumOfTextView(minHeight: Int): Int {
    var n = 2
    while (70 * n - 30 < minHeight) { n++ }
    return 5 * n - 6 // 4 + 5 * (n - 2)
}

private fun updateAppWidget(context: Context, appWidgetManager: AppWidgetManager, appWidgetId: Int, numOfTextView: Int, data: List<SpannableString>) {
    val useMore: Boolean = numOfTextView < data.size
    val remoteViews = RemoteViews(context.packageName, R.layout.appwidget_reminders)
    val textViews = if (useMore) {
        List(numOfTextView - 1) {
            RemoteViews(context.packageName, R.layout.appwidget_reminders_row)
        }.plusElement(
            RemoteViews(context.packageName, R.layout.appwidget_reminders_row_last)
        )
    } else {
        List(numOfTextView) {
            RemoteViews(context.packageName, R.layout.appwidget_reminders_row)
        }
    }
    remoteViews.removeAllViews(R.id.linearLayout)
    textViews.forEach { remoteViews.addView(R.id.linearLayout, it) }

    textViews.forEachIndexed { index, remoteView ->
        if (index < data.size) {
            if (index == textViews.size - 1 && useMore) {
                remoteView.setTextViewText(R.id.textView, context.getString(R.string.label_widget_more, data.size - textViews.size + 1))
            } else {
                remoteView.setTextViewText(R.id.textView, data[index])
            }
        } else {
            remoteView.setTextViewText(R.id.textView, "")
        }
    }

    val intent = Intent(context, HowmmMain::class.java)
    val flag = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
        PendingIntent.FLAG_IMMUTABLE or PendingIntent.FLAG_UPDATE_CURRENT
    } else {
        PendingIntent.FLAG_UPDATE_CURRENT
    }
    val pendingIntent = PendingIntent.getActivity(context, 0, intent, flag)
    remoteViews.setOnClickPendingIntent(R.id.linearLayout, pendingIntent)
    appWidgetManager.updateAppWidget(appWidgetId, remoteViews)
}

class HowmmReminderAppWidget : AppWidgetProvider() {
    override fun onUpdate(context: Context, appWidgetManager: AppWidgetManager, appWidgetIds: IntArray) {
        super.onUpdate(context, appWidgetManager, appWidgetIds)

        appWidgetIds.forEach { appWidgetId ->
            val options = appWidgetManager.getAppWidgetOptions(appWidgetId)
            onAppWidgetOptionsChanged(context, appWidgetManager, appWidgetId, options)
        }
    }

    private val mainScope = MainScope()

    override fun onAppWidgetOptionsChanged(
            context: Context,
            appWidgetManager: AppWidgetManager,
            appWidgetId: Int,
            newOptions: Bundle) {
        mainScope.launch(Dispatchers.Main) {
            var waitCount = 0
            val maxWait = 9
            val rootDir = File(howmmPrefs.rootDir)
            while (!rootDir.canRead()) {
                Log.i("Howmm", "waiting for the data dir ... $waitCount")
                delay(3000)

                waitCount++
                if (waitCount > maxWait) {
                    Log.e("Howmm", "unable to read the data dir!")
                }
            }
//            while (Environment.MEDIA_MOUNTED != Environment.getExternalStorageState()) {
//                Log.i("Howmm", "waiting mount SD ... $waitCount")
//                delay(3000)
//
//                waitCount++
//                if (waitCount > maxWait) {
//                    Log.e("Howmm", "no SD card found!")
//                }
//            }
            val reminders = grepReminders(context)
            val spannableStrings = reminders.map {
                val rItem = it.reminder
                if (rItem != null) {
                    TextSpanner.spanReminderAdapter(it.text, rItem.type, rItem.priority)
                } else {
                    SpannableString("")
                }
            }

            val minHeight = newOptions.getInt(AppWidgetManager.OPTION_APPWIDGET_MIN_HEIGHT)
            updateAppWidget(context, appWidgetManager, appWidgetId, calcNumOfTextView(minHeight), spannableStrings)
        }
    }

    override fun onReceive(context: Context, intent: Intent) {
        if (intent.action == ACTION_HOWMM_WIDGET_SENDDATA) {
            val spannableStrings = intent.getStringArrayExtra(EXTRA_HOWMM_WIDGET_SENDDATA)?.mapNotNull {
                val grepItemStr = it.split(",")
                if (grepItemStr.size != 3) {
                    Log.e("Howmm", "widget doUpdate(): Broken data")
                    null
                } else {
                    TextSpanner.spanReminderAdapter(grepItemStr[0], ReminderItem.ReminderType.valueOf(grepItemStr[1]), Integer.parseInt(grepItemStr[2]))
                }
            } ?: listOf()
            val manager = AppWidgetManager.getInstance(context)
            manager.getAppWidgetIds(ComponentName(context, HowmmReminderAppWidget::class.java.name))
                .forEach { appWidgetId ->
                    val minHeight = manager.getAppWidgetOptions(appWidgetId)
                            .getInt(AppWidgetManager.OPTION_APPWIDGET_MIN_HEIGHT)
                    updateAppWidget(context, manager, appWidgetId, calcNumOfTextView(minHeight), spannableStrings)
                }
        }

        super.onReceive(context, intent)
    }

    override fun onDisabled(context: Context) {
        mainScope.cancel()
    }

    companion object {
        const val ACTION_HOWMM_WIDGET_SENDDATA = "jp.deadend.noname.howmm.HowmmAppWidget.ACTION_HOWMM_WIDGET_SENDDATA"
        const val EXTRA_HOWMM_WIDGET_SENDDATA = "jp.deadend.noname.howmm.HowmmAppWidget.EXTRA_HOWMM_WIDGET_SENDDATA"
    }
}