package jp.deadend.noname.howmm

import android.app.Activity
import android.app.SearchManager
import android.content.Intent
import android.os.Bundle
import android.view.*
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.SearchView
import android.widget.AdapterView
import android.widget.AdapterView.OnItemSelectedListener
import android.widget.ArrayAdapter
import android.widget.Space
import android.widget.Spinner
import androidx.activity.result.contract.ActivityResultContracts
import androidx.window.layout.WindowMetricsCalculator
import jp.deadend.noname.howmm.databinding.ActivityResultsBinding
import kotlinx.coroutines.*

class HowmmResults : AppCompatActivity(), GrepView.Callback {
    private lateinit var binding: ActivityResultsBinding
    private lateinit var mViewerPopup: TextViewerPopup
    private var mSearchView: SearchView? = null
    private var mSearchResult: List<GrepItem>? = null
    private var mRegex: Regex? = null
    private var mComeFrom: Regex? = null
    private var mMinTime: Long = 0
    private var mSortOrder = SORT_BY_DATE
    private lateinit var mQueryStr: String
    private val textEditorActivity = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
        if (it.resultCode == Activity.RESULT_OK) { doSearch() }
    }
    private val mainScope = MainScope()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        binding = ActivityResultsBinding.inflate(layoutInflater)
        setContentView(binding.root)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        title = ""

        mQueryStr = intent?.extras?.getString(SearchManager.QUERY) ?: ""

        val spinnerDays = binding.toolbarTools.findViewById<Spinner>(R.id.spinnerDays)
        spinnerDays.adapter = ArrayAdapter.createFromResource(
                this, R.array.label_days, android.R.layout.simple_spinner_dropdown_item)
        spinnerDays.onItemSelectedListener = object : OnItemSelectedListener {
            override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
                val now = System.currentTimeMillis()
                mMinTime = when (position) {
                    0 -> 0
                    1 -> now - 1000L * 60L * 60L * 24L * 30L  // 1 month
                    2 -> now - 1000L * 60L * 60L * 24L * 182L // 6 months
                    3 -> now - 1000L * 60L * 60L * 24L * 365L // 1 year
                    else -> mMinTime
                }
                setGrepView()
            }

            override fun onNothingSelected(parent: AdapterView<*>) {}
        }
        val spinnerSort = binding.toolbarTools.findViewById<Spinner>(R.id.spinnerSort)
        spinnerSort.adapter = ArrayAdapter.createFromResource(
                this, R.array.label_sort_types, android.R.layout.simple_spinner_dropdown_item)
        spinnerSort.onItemSelectedListener = object : OnItemSelectedListener {
            override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
                mSortOrder = when (position) {
                    0 -> SORT_BY_DATE
                    1 -> SORT_BY_SUMMARY
                    2 -> SORT_BY_DATE_REVERSE
                    3 -> SORT_BY_SUMMARY_REVERSE
                    4 -> SORT_SHUFFLE
                    else -> mSortOrder
                }
                setGrepView()
            }

            override fun onNothingSelected(parent: AdapterView<*>) {}
        }

        binding.grepView.setCallback(this)
        binding.grepView.onItemSelectedListener = object : OnItemSelectedListener {
            override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
                onGrepItemClicked(position)
            }

            override fun onNothingSelected(parent: AdapterView<*>) {}
        }
        val footerView = Space(this)
        footerView.minimumHeight = 0
        binding.grepView.addFooterView(footerView, null, false)

        binding.floatingActionButton.setOnClickListener {
            if (mViewerPopup.isShowing) { mViewerPopup.dismiss() }
            val intent = Intent(this@HowmmResults, TextEditor::class.java).apply {
                putExtra(TextEditor.KEY_PATHNAME, null as String?)
                putExtra(TextEditor.KEY_TITLE, "")
            }
            textEditorActivity.launch(intent)
        }

        doSearch()
    }

    override fun onAttachedToWindow() {
        mViewerPopup = TextViewerPopup(this, null)
        super.onAttachedToWindow()
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.menu_results, menu)
        mSearchView = menu.findItem(R.id.menu_search).actionView as SearchView
        mSearchView?.apply {
            isIconified = false
            clearFocus()
            val width = WindowMetricsCalculator.getOrCreate()
                            .computeCurrentWindowMetrics(this@HowmmResults)
                            .bounds.width()
            maxWidth = (0.5 * width).toInt() // workaround
            if (mQueryStr.isNotEmpty()) { setQuery(mQueryStr, false) }
            setOnQueryTextListener(object : SearchView.OnQueryTextListener {
                override fun onQueryTextSubmit(query: String): Boolean {
                    if (mViewerPopup.isShowing) { mViewerPopup.dismiss() }
                    val intent = Intent(this@HowmmResults, HowmmResults::class.java).apply {
                        action = Intent.ACTION_SEARCH
                        putExtra(SearchManager.QUERY, query.trim())
                        flags = Intent.FLAG_ACTIVITY_CLEAR_TOP
                    }
                    startActivity(intent)
                    return true
                }

                override fun onQueryTextChange(s: String) = false
            })
            setOnQueryTextFocusChangeListener { _, hasFocus ->
                if (hasFocus && mViewerPopup.isShowing) {
                    mViewerPopup.dismiss()
                }
            }
        }

        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.menu_open_tools -> {
                if (binding.toolbarTools.visibility == View.GONE) {
                    binding.toolbarTools.visibility = View.VISIBLE
                    binding.toolbarToolsShadow.visibility = View.VISIBLE
                    item.setIcon(R.drawable.ic_cancel_24dp)
                } else {
                    binding.toolbarTools.visibility = View.GONE
                    binding.toolbarToolsShadow.visibility = View.GONE
                    item.setIcon(R.drawable.ic_arrow_drop_down_circle_24dp)
                }
            }
            R.id.menu_results_reload -> {
                if (mQueryStr.isEmpty()) { clearCacheFile(TITLE_CACHE_FILE) }
                doSearch()
            }
            android.R.id.home -> onBackPressed()
            else -> return super.onOptionsItemSelected(item)
        }

        return true
    }

    private fun doSearch() {
        mSearchResult = null
        invokeTask(mQueryStr)
        mSearchView?.setQuery(mQueryStr, false)
    }

    private fun invokeTask(query: String) {
        mainScope.launch(Dispatchers.Main) {
            binding.progressBar.visibility = View.VISIBLE
            binding.grepView.visibility = View.INVISIBLE

            mRegex = null
            mComeFrom = null
            val results: List<GrepItem>
            when {
                query.isEmpty() -> results = grepTitles()
                query.startsWith("#") -> results = grepHashtags(query)
                else -> {
                    val ignoreCase = howmmPrefs.ignoreCase
                    if (howmmPrefs.useRegex) {
                        val regex = if (ignoreCase) {
                            Regex(query, RegexOption.IGNORE_CASE)
                        } else {
                            Regex(query)
                        }
                        results = grepRegex(regex)
                        mRegex = regex
                    } else {
                        val escaped = escapeMetaChar(query)
                        val queryList = escaped.split("[ 　\\t]+".toRegex())
                        val queryStr =
                                if (queryList.size <= 1) escaped else queryList.joinToString(separator = "|")
                        mComeFrom = Regex("""<<<[ 　\t]*($queryStr)\b""")
                        // mRegex is for highlighting
                        mRegex = if (ignoreCase) {
                            Regex(queryStr, RegexOption.IGNORE_CASE)
                        } else {
                            Regex(queryStr)
                        }
                        results = grepMultipleWords(queryList, ignoreCase)
                    }
                }
            }

            if (results.isNotEmpty()) {
                mSearchResult = results
                setGrepView()

                binding.progressBar.visibility = View.GONE
                binding.grepView.visibility = View.VISIBLE
            } else {
                showErrorDialog()
            }
        }
    }

    private fun escapeMetaChar(str: String): String {
        val metachars = ".^$[]*+?|()\\"
        val newstr = StringBuilder()

        val len = str.length
        for (i in 0 until len) {
            val c = str[i]
            if (metachars.contains(c)) {
                newstr.append('\\')
            }
            newstr.append(c)
        }

        return newstr.toString()
    }

    override fun dispatchKeyEvent(event: KeyEvent): Boolean {
        val keycode = event.keyCode
        if (event.action == KeyEvent.ACTION_UP && mViewerPopup.isShowing) {
            when (keycode) {
                KeyEvent.KEYCODE_SPACE -> {
                    if (event.isShiftPressed || event.isAltPressed) {
                        mViewerPopup.pageUp()
                    } else {
                        mViewerPopup.pageDown()
                    }
                    return true
                }
                KeyEvent.KEYCODE_PAGE_DOWN -> {
                    mViewerPopup.pageDown()
                    return true
                }
                KeyEvent.KEYCODE_PAGE_UP -> {
                    mViewerPopup.pageUp()
                    return true
                }
                else -> return super.dispatchKeyEvent(event)
            }
        }

        return super.dispatchKeyEvent(event)
    }

    override fun onGrepItemClicked(position: Int) {
        mViewerPopup.show(binding.linearLayoutMain)
        val item = binding.grepView.adapter.getItem(position) as GrepItem
        mViewerPopup.loadFile(mRegex, item.file, item.linenumber)
    }

    override fun onGrepItemLongClicked(position: Int): Boolean {
        val item = binding.grepView.adapter.getItem(position) as GrepItem

        if (mViewerPopup.isShowing) { mViewerPopup.dismiss() }
        val intent = Intent(this@HowmmResults, TextEditor::class.java).apply {
            putExtra(TextEditor.KEY_PATHNAME, item.file.absolutePath)
            putExtra(TextEditor.KEY_LINENO, item.linenumber)
        }
        textEditorActivity.launch(intent)

        return true
    }

    private fun setGrepView() {
        val result = mSearchResult ?: return
        if (result.isEmpty()) { return }

        if (mViewerPopup.isShowing) { mViewerPopup.dismiss() }

        val resultToDisplay = result.filter { it.file.lastModified() >= mMinTime }.toMutableList()

        if (mSortOrder == SORT_SHUFFLE) {
            resultToDisplay.shuffle()
        } else {
            val comefromComparator = mComeFrom?.let { regex ->
                compareByDescending<GrepItem> { regex.matches(it.text) }
            }
            val comparator = when (mSortOrder) {
                SORT_BY_SUMMARY -> {
                    comefromComparator?.let { cmp ->
                        cmp.thenByDescending { it.text }.thenBy { it.file.lastModified() }
                    } ?: compareByDescending<GrepItem> { it.text }.thenBy { it.file.lastModified() }
                }
                SORT_BY_SUMMARY_REVERSE -> {
                    comefromComparator?.let { cmp ->
                        cmp.thenBy { it.text }.thenBy { it.file.lastModified() }
                    } ?:compareBy<GrepItem> { it.text }.thenBy { it.file.lastModified() }
                }
                SORT_BY_DATE_REVERSE -> {
                    comefromComparator?.let { cmp ->
                        cmp.thenBy { it.file.lastModified() }.thenBy { it.linenumber }
                    } ?: compareBy<GrepItem> { it.file.lastModified() }.thenBy { it.linenumber }
                }
                else -> { // SORT_BY_DATE，デフォルト
                    comefromComparator?.let { cmp ->
                        cmp.thenByDescending { it.file.lastModified() }.thenBy { it.linenumber }
                    } ?: compareByDescending<GrepItem> { it.file.lastModified() }.thenBy { it.linenumber }
                }
            }
            resultToDisplay.sortWith(comparator)
        }

        val context = applicationContext
        val adapter = GrepView.GrepAdapter(context, resultToDisplay)
        adapter.setFormat(mRegex, howmmPrefs.fontSize)
        binding.grepView.adapter = adapter
    }

    override fun onPause() {
        if (isFinishing) { mainScope.cancel() }
        super.onPause()
    }

    override fun onDetachedFromWindow() {
        if (mViewerPopup.isShowing) { mViewerPopup.dismiss() }
        super.onDetachedFromWindow()
    }

    override fun onBackPressed() {
        if (mViewerPopup.isShowing) {
            mViewerPopup.dismiss()
        } else {
            super.onBackPressed()
        }
    }

    private fun showErrorDialog() {
        val dialog = SimpleMessageDialogFragment.newInstance(getString(R.string.label_not_found))
        dialog.setListener(
                object : SimpleMessageDialogFragment.Listener {
                    override fun onClick() { finish() }
                })
        dialog.show(supportFragmentManager, "dialog")
    }

    companion object {
        private const val SORT_BY_DATE = 0
        private const val SORT_BY_DATE_REVERSE = 1
        private const val SORT_BY_SUMMARY = 2
        private const val SORT_BY_SUMMARY_REVERSE = 3
        private const val SORT_SHUFFLE = 4
    }
}
