package jp.deadend.noname.howmm

import android.content.Context
import android.util.AttributeSet
import android.view.MotionEvent
import android.widget.ListView

open class ScrollInhibitableListView : ListView {
    private var mPosition: Int = 0
    private var isScrollable = true

    constructor(context: Context) : super(context)
    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)
    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

    fun enableScroll(value: Boolean) {
        isScrollable = value
    }

    override fun dispatchTouchEvent(ev: MotionEvent): Boolean {
        if (!isScrollable) {
            val actionMasked = ev.actionMasked and MotionEvent.ACTION_MASK

            if (actionMasked == MotionEvent.ACTION_DOWN) {
                // Record the position the list the touch landed on
                mPosition = pointToPosition(ev.x.toInt(), ev.y.toInt())
                return super.dispatchTouchEvent(ev)
            }

            if (actionMasked == MotionEvent.ACTION_MOVE) {
                // Ignore move events
                return true
            }

            if (actionMasked == MotionEvent.ACTION_UP) {
                // Check if we are still within the same view
                if (pointToPosition(ev.x.toInt(), ev.y.toInt()) == mPosition) {
                    super.dispatchTouchEvent(ev)
                } else {
                    // Clear pressed state, cancel the action
                    isPressed = false
                    invalidate()
                    return true
                }
            }
        }

        return super.dispatchTouchEvent(ev)
    }
}
