package jp.deadend.noname.howmm

import android.content.Context
import android.text.method.LinkMovementMethod
import android.text.util.Linkify
import android.util.AttributeSet
import android.util.Log
import android.widget.TextView
import android.widget.ScrollView

import java.io.File
import java.io.IOException
import java.util.regex.Pattern

class TextFileViewer : ScrollView {
    companion object {
        private val URL_PATTERN = Pattern.compile("(https?|ftp)://([\\w-]+\\.)+[\\w-]+(/[\\w-./?%&=]*)?")
    }

    constructor(context: Context) : super(context)
    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)
    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

    private val mAppContext = context.applicationContext
    private var mFile: File? = null
    private var mRegex: Regex? = null
    private val mTextView = TextView(context)

    init {
        mTextView.linksClickable = true
        mTextView.setTextColor(-0x1000000)
        mTextView.textSize = howmmPrefs.fontSize.toFloat()
        val density = context.resources.displayMetrics.density
        mTextView.setPadding((4 * density).toInt(), 0, (4 * density).toInt(), 0)
        addView(mTextView)
    }

    fun loadFile(file: File): Boolean {
        if (file.exists()) {
            val lines = try {
                file.bufferedReader(charset(howmmPrefs.encoding)).use { it.readLines() }
            } catch (e: IOException) {
                Log.e("Howmm", "TextFileViewer#loadFile(): File IO error " + file.absolutePath)
                null
            }
            lines?.let {
                mTextView.text = TextSpanner.spanTextViewer(it, mRegex, mAppContext)
                mTextView.movementMethod = LinkMovementMethod.getInstance()
                mFile = file
                Linkify.addLinks(mTextView, Linkify.EMAIL_ADDRESSES)
                Linkify.addLinks(mTextView, URL_PATTERN, null)

                return true
            }
        }

        return false
    }

    fun reloadFile(): Boolean {
        val file = mFile
        return if (file != null) loadFile(file) else false
    }

    fun setRegex(regex: Regex?) {
        mRegex = regex
    }

    fun scrollByLine(line: Int): Boolean {
        val offset = mTextView.text.getOffsetForLine(line)
        if (offset == -1) { return false }

        val layout = mTextView.layout
        layout?.let {
            scrollTo(0, it.getLineTop(it.getLineForOffset(offset)))
            return true
        }
        return false
    }
}
