package jp.deadend.noname.howmm

import android.content.Context
import android.content.res.Configuration
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import android.widget.PopupWindow
import android.widget.TextView
import androidx.core.content.res.ResourcesCompat

import java.io.File

class TextViewerPopup(context: Context, private val mResizeView: View?) : PopupWindow(context) {
    private val mTextFileViewer: TextFileViewer
    private val mFilenameView: TextView
    private val mMaximizeButton: ImageView
    private var maximized = false
    private var mOrientation = Configuration.ORIENTATION_PORTRAIT
    private var mScreenWidth = 0
    private var mScreenHeight = 0

    init {
        mOrientation = context.resources.configuration.orientation

        val popupView = LayoutInflater.from(context).inflate(R.layout.textviewer, null)
        mTextFileViewer = popupView.findViewById(R.id.textFileViewer) as TextFileViewer
        mFilenameView = popupView.findViewById(R.id.textViewFileName) as TextView
        popupView.findViewById<ImageView>(R.id.buttonClose).setOnClickListener {
            if (mResizeView != null) { mResizeView.minimumHeight = 0 }
            dismiss()
        }
        mMaximizeButton = popupView.findViewById(R.id.buttonMaximize) as ImageView
        mMaximizeButton.setOnClickListener {
            if (maximized) {
                unmaximize()
            } else {
                maximize()
            }
            maximized = !maximized
        }

        setBackgroundDrawable(ResourcesCompat.getDrawable(
            context.resources, R.drawable.popup_background, null)
        )
        contentView = popupView
    }

    fun loadFile(regex: Regex?, file: File, line: Int) {
        mFilenameView.text = file.absolutePath

        mTextFileViewer.setRegex(regex)
        mTextFileViewer.loadFile(file)
        mTextFileViewer.scrollByLine(line)
    }

    fun reloadFile() {
        mTextFileViewer.reloadFile()
    }

    fun pageDown() {
        mTextFileViewer.pageScroll(View.FOCUS_DOWN)
    }

    fun pageUp() {
        mTextFileViewer.pageScroll(View.FOCUS_UP)
    }

    private fun maximize() {
        if (mOrientation == Configuration.ORIENTATION_LANDSCAPE) {
            mMaximizeButton.setImageResource(R.drawable.ic_right_arrow_24dp)
        } else {
            mMaximizeButton.setImageResource(R.drawable.ic_down_arrow_24dp)
        }
        update(mScreenWidth - 8, mScreenHeight - 8)
    }

    private fun unmaximize() {
        if (mOrientation == Configuration.ORIENTATION_LANDSCAPE) {
            update(2 * mScreenWidth / 3, mScreenHeight - 8)
            mMaximizeButton.setImageResource(R.drawable.ic_left_arrow_24dp)
        } else {
            update(mScreenWidth - 8, mScreenHeight / 2)
            mMaximizeButton.setImageResource(R.drawable.ic_up_arrow_24dp)
        }
    }

    fun show(parent: View) {
        if (!isShowing) {
            mScreenWidth = parent.width
            mScreenHeight = parent.height

            var gravity = Gravity.BOTTOM
            if (mOrientation == Configuration.ORIENTATION_LANDSCAPE) {
                gravity = gravity or Gravity.RIGHT
                width = 2 * mScreenWidth / 3
                height = mScreenHeight - 8
                mMaximizeButton.setImageResource(R.drawable.ic_left_arrow_24dp)
                animationStyle = R.style.animation_popup_horizontal
            } else {
                gravity = gravity or Gravity.CENTER_HORIZONTAL
                width = mScreenWidth - 8
                height = mScreenHeight / 2
                mMaximizeButton.setImageResource(R.drawable.ic_up_arrow_24dp)
                animationStyle = R.style.animation_popup_vertical
                if (mResizeView != null) {
                    mResizeView.minimumHeight = mScreenHeight / 2
                }
            }
            maximized = false
            showAtLocation(parent, gravity, 0, 0)
        }
    }
}
